package MojoMojo::View::JSON;
use parent qw( Catalyst::View::JSON );

__PACKAGE__->config->{expose_stash}='json';


=head1 NAME

MojoMojo::View::JSON - Catalyst View

=head1 DESCRIPTION

Catalyst View.

=head1 AUTHOR

Sachin Sebastian <sachinjsk at cpan.org>

=head1 LICENSE

This library is free software. You can redistribute it and/or modify
it under the same terms as Perl itself.

=cut

1;
