.progressbar {
    font-family: Arial;
    font-size: 10pt;
    text-align: center;
}

.progressbar table {
}

.progressbar th {
    font-weight: bold;
    text-align: right;
}

.progressbar td {
    text-align: left;
}

.finished {
    color: #667799;
}

.aborted {
    color: red;
}

.progressbar .bar {
    position: relative;
    background-color: lightgrey;
    border: 2px solid #667799;
    height: 20px;
    width: 380px;
}

.progressbar .bar .amount {
    background-color: #667799;
    width: 0px;
    height: 20px
}

.progressbar .bar .percent {
    position: absolute;
    top: 0px;
    left: 0px;
    width: 100%;
    height: 20px;
    z-index: 1000;
    text-align: center;
    font-family: Arial;
    font-size: 18px;
    font-weight: bold;
}
