body { line-height: 1.5; font-family: Helvetica Neue, Arial, Helvetica, sans-serif; color: #000; background: none; font-size: 10pt; }

.container { background: none; }

hr { background: #ccc; color: #ccc; width: 100%; height: 2px; margin: 2em 0; padding: 0; border: none; }
hr.space { background: #fff; color: #fff; }

h1, h2, h3, h4, h5, h6 { font-family: Helvetica Neue, Arial, Helvetica, sans-serif; }

code { font-size: .9em; font-family: 'andale mono', 'lucida console', monospace; }

img { display: inline; float: left; margin: 1.5em 1.5em 1.5em 0; }

a img { border: none; }
a:link, a:visited { background: transparent; font-weight: 700; text-decoration: underline; }

p img.top { margin-top: 0; }

blockquote { margin: 1.5em; padding: 1em; font-style: italic; font-size: .9em; }

.small { font-size: .9em; }

.large { font-size: 1.1em; }

.quiet { color: #999; }

.hide { display: none; }
