package MojoMojo::Schema::Base::ResultSet;

use strict;
use warnings;
use parent qw/DBIx::Class::ResultSet/;

=head1 NAME

MojoMojo::Schema::Base::ResultSet - base class for ResultSet classes

=head1 DESCRIPTION

Base class for all resultsets below the MojoMojo::Schema::ResultSet::* namespace.

=head1 LICENSE

This library is free software. You can redistribute it and/or modify
it under the same terms as Perl itself.

=cut

1;
