@import blueprint.sass

+blueprint-ie 

// Note :
//
// Compass will center text to fix IE6 container centering.
// This means all your texts will be centered under all version of IE by default.
// Please don't forget to restore the correct behavior to your main container (but not the body tag!)
//
// Example :
//
// .container, .footer
//   :text-align left

