html, body { margin: 0; padding: 0; border: 0; font-weight: inherit; font-style: inherit; font-size: 100%; font-family: inherit; vertical-align: baseline; }

div, span, object, iframe, h1, h2, h3, h4, h5, h6, p, pre, a, abbr, acronym, address, code, del, dfn, em, img, dl, dt, dd, ol, ul, li, fieldset, form, label, legend, caption, tbody, tfoot, thead, tr { margin: 0; padding: 0; border: 0; font-weight: inherit; font-style: inherit; font-size: 100%; font-family: inherit; vertical-align: baseline; }

blockquote, q { margin: 0; padding: 0; border: 0; font-weight: inherit; font-style: inherit; font-size: 100%; font-family: inherit; vertical-align: baseline; quotes: "" ""; }
blockquote:before, q:before, blockquote:after, q:after { content: ""; }

th, td, caption { margin: 0; padding: 0; border: 0; font-weight: inherit; font-style: inherit; font-size: 100%; font-family: inherit; vertical-align: baseline; text-align: left; font-weight: normal; vertical-align: middle; }

table { margin: 0; padding: 0; border: 0; font-weight: inherit; font-style: inherit; font-size: 100%; font-family: inherit; vertical-align: baseline; border-collapse: separate; border-spacing: 0; vertical-align: middle; }

a img { border: none; }

body.blueprint { line-height: 1.5; font-family: Helvetica Neue, Arial, Helvetica, sans-serif; color: #333333; font-size: 75%; }
body.blueprint h1 { font-weight: normal; color: #222222; font-size: 3em; line-height: 1; margin-bottom: 0.5em; }
body.blueprint h1 img { margin: 0; }
body.blueprint h2 { font-weight: normal; color: #222222; font-size: 2em; margin-bottom: 0.75em; }
body.blueprint h3 { font-weight: normal; color: #222222; font-size: 1.5em; line-height: 1; margin-bottom: 1em; }
body.blueprint h4 { font-weight: normal; color: #222222; font-size: 1.2em; line-height: 1.25; margin-bottom: 1.25em; }
body.blueprint h5 { font-weight: normal; color: #222222; font-size: 1em; font-weight: bold; margin-bottom: 1.5em; }
body.blueprint h6 { font-weight: normal; color: #222222; font-size: 1em; font-weight: bold; }
body.blueprint h2 img, body.blueprint h3 img, body.blueprint h4 img, body.blueprint h5 img, body.blueprint h6 img { margin: 0; }
body.blueprint p { margin: 0 0 1.5em; }
body.blueprint p img.left { display: inline; float: left; margin: 1.5em 1.5em 1.5em 0; padding: 0; }
body.blueprint p img.right { display: inline; float: right; margin: 1.5em 0 1.5em 1.5em; padding: 0; }
body.blueprint a { text-decoration: underline; color: #000099; }
body.blueprint a:visited { color: #000066; }
body.blueprint a:focus { color: black; }
body.blueprint a:hover { color: black; }
body.blueprint a:active { color: #cc0099; }
body.blueprint blockquote { margin: 1.5em; color: #666; font-style: italic; }
body.blueprint strong { font-weight: bold; }
body.blueprint em { font-style: italic; }
body.blueprint dfn { font-style: italic; font-weight: bold; }
body.blueprint sup, body.blueprint sub { line-height: 0; }
body.blueprint abbr, body.blueprint acronym { border-bottom: 1px dotted #666; }
body.blueprint address { margin: 0 0 1.5em; font-style: italic; }
body.blueprint del { color: #666; }
body.blueprint pre { margin: 1.5em 0; white-space: pre; }
body.blueprint pre, body.blueprint code, body.blueprint tt { font: 1em 'andale mono', 'lucida console', monospace; line-height: 1.5; }
body.blueprint li ul, body.blueprint li ol { margin: 0 1.5em; }
body.blueprint ul { margin: 0 1.5em 1.5em 1.5em; list-style-type: disc; }
body.blueprint ol { margin: 0 1.5em 1.5em 1.5em; list-style-type: decimal; }
body.blueprint dl { margin: 0 0 1.5em 0; }
body.blueprint dl dt { font-weight: bold; }
body.blueprint dd { margin-left: 1.5em; }
body.blueprint table { margin-bottom: 1.4em; width: 100%; }
body.blueprint th { font-weight: bold; }
body.blueprint thead th { background: #c3d9ff; }
body.blueprint th, body.blueprint td, body.blueprint caption { padding: 4px 10px 4px 5px; }
body.blueprint tr.even td { background: #e5ecf9; }
body.blueprint tfoot { font-style: italic; }
body.blueprint caption { background: #eee; }
body.blueprint .quiet { color: #666666; }
body.blueprint .loud { color: #111111; }

body { background: #083e61 url(images/bg.gif) repeat-x; color: #FFFFFF; }

#container { min-width: 950px; width: 80%; margin: 0 auto; overflow: hidden; display: inline-block; }
#container { display: block; }
#container h1, #container h2, #container h3, #container h4 { color: #1050A7; margin: 10px 0 10px  0; }
#container h1 { font-size: 1.4em; }
#container h2 { font-size: 1.3em; }
#container h3 { font-size: 1.2em; }
#container h4 { font-size: 1.1em; }
#container p { margin: 5px; }

#header, #breadcrumbs, #toplinks, #pagenav, #content, #showtag, #tags, #bottomnav, #powered { display: inline; float: left; overflow: hidden; width: 100%; margin-right: 0.833em; margin: 10px; }

#content_preview { display: inline; float: left; overflow: hidden; width: 41%; margin-right: 0.833em; }

#edit_form { display: inline; float: left; overflow: hidden; width: 41%; margin-right: 0; }

#toplinks, #pagenav, #pageoptions, #bottomnav { margin: 0; padding: 0; border: 0; outline: 0; overflow: hidden; display: inline-block; border: 0px solid #777; padding: 3px; font-family: sans-serif; }
#toplinks, #pagenav, #pageoptions, #bottomnav { display: block; }
#toplinks li, #pagenav li, #pageoptions li, #bottomnav li { list-style-type: none; margin-left: 0px; white-space: nowrap; display: inline; float: left; padding-left: 4px; padding-right: 4px; }
#toplinks li.first, #pagenav li.first, #pageoptions li.first, #bottomnav li.first { padding-left: 0px; }
#toplinks li.last, #pagenav li.last, #pageoptions li.last, #bottomnav li.last { padding-right: 0px; }
#toplinks a, #pagenav a, #pageoptions a, #bottomnav a { -moz-border-radius: 5px; -webkit-border-radius: 5px; border: 1px solid #000; color: #000; padding: 2px; margin: 5px 0px 5px 0px; background-color: #939396; text-decoration: none; }

#header h1 { color: #ADD8E6; }
#header h1:before { content: url(images/catalyst_logo.png); }

#breadcrumbs { padding: 10px; }
#breadcrumbs a { color: #6198E2; }

#content { background: #aaa; color: #000; border: 1px solid #bbb; }
#content a { text-decoration: none; color: #990000; }
#content a:hover { text-decoration: underline; }
#content a:visited { color: #990000; }
#content a:focus { color: #990000; }
#content a:hover { color: #990000; }
#content a:active { color: #990000; }
#content #current { margin: 10px; padding: 0 2em 0 0; }
#content #current a { color: #900; }
#content #current p { margin: 0 0 10px 0; }
#content #current pre { -moz-border-radius: 10px; color: #F5DEB3; background: #083e61; padding: 10px; margin: 10px; border: 1px solid #000; }
#content #current em { font-style: italic; }
#content #current img { margin: 10px; float: left; }
#content pre { color: #000; }
#content .toc { background-color: #788086; border: 1px solid #000; padding: 10px; margin: 10px; float: right; position: relative; }
#content .toc ul { margin: 0; padding: 0; margin-left: +1em; }
#content .toc li { margin: 0; padding: 0; font-size: 1em; }

#comments { margin: 10px; }
#comments h4 { color: #0F7815; }
#comments .comment { -moz-border-radius: 5px; -webkit-border-radius: 5px; border: 1px solid #000; color: #000; padding: 2px; margin: 5px 0px 5px 0px; background: #DBDBE2; }

table { border: 1px solid #000; padding: 10px; background: #9C9C9C; }
table thead tr { background: #8C8C8C; }

.filename { color: #0B5F18; }

#nonexistent_pages { background-color: #E97064; }

#powered img { float: right; position: relative; margin: 10px; }

.byline { color: #FFF; }
