/* @override http://localhost:3000/.static/themes/default/theme.css */
/* @group Document Structure */
body {
    background: #99c8c7;
    color: #424242;
    margin: 0;
    font-size: 0.9em;
    font-family: Helvetica, Geneva, sans-serif;
} 

.container {
    max-width: 960px;
    text-align: left;
    margin: 0 auto 0;
    border-right: 3px solid #a5cdcb;
    border-left: 3px solid #a5cdcb;
} 

.maximized-container {
    max-width: 95%;
    width: 95%;
} 

#header {
    display:none;
} 

#head {
    height:90px;
    background:#333;
}

#head h1 {
    margin:0;
    text-indent: -3333em;
    background:transparent url(images/onion-madrid.png) no-repeat 30px 11px;
}

#head h1 > a {
    display: block;
    width: 130px;
    height: 90px;
    text-decoration: none;
}

#breadcrumbs {
    background: #555;
    height: 16px;
    font-size: 1em;
    border-top: 2px solid #444;
    border-bottom: 1px solid #ccc;
    color: #e3e3e3;
    margin: 0;
    padding: 6px 8px;
} 

#breadcrumbs a {
    padding: 2px 7px;
    color: #fffff3;
    text-decoration: none;
} 

#breadcrumbs a:hover {
    color: #ffffff;
    background-color: #999;
    text-decoration: none;
    -moz-border-radius: 3px;
    -webkit-border-radius: 3px;
}

pre {
    background-color:#0c0c0c;
    border:6px solid #ddd;
    color:#fff;
    font-size: 1.1em;
    -moz-border-radius: 6px;
    -webkit-border-radius: 6px;
}
pre b { color: yellow; }
pre span.kateDataType { color: #00cccc; }
pre span.kateOperator { color: yellow; }
pre span.kateString { color: #cc00c2; }
pre span.kateFloat { color: #cc00c2; }
pre span.kateComment { color: #00cccc; font-style: italic; }
pre span.kateFunction { color: #00cc00; }

div#toplinks {
    position:absolute;
    top:20px;
    left:60%;
    border:2px solid #222;
    max-width:420px;
    font-size: .9em;
    background: #e0e0e0;
    color: #000000;
    padding: 4px 10px;
}

div#toplinks ul li, div#bottomnav ul li {
    display: block;
    float: left;
}

div#bottomnav ul li {
    min-height: 1.4em;
    _height: 1.4em;
}

div#toplinks ul li a, div#bottomnav ul li a, div#bottomnav ul li span.title {
    text-decoration: none;
    margin: 0px 0 0 5px;
    padding: 1px 2px;
}

div#toplinks ul {
    margin:6px 0 0 0;
}

div#toplinks ul li a {
    padding: 2px;
}

div#toplinks ul li a:hover, div#bottomnav ul li a:hover {
    background-color: #fff;
    color: #333;
}

ul#toplinks-for-user { 
    position:absolute;
    top:-22px;
    right:0; 
}
ul#toplinks-for-user li {
    margin-right:5px;
}
ul#toplinks-for-user li.register {
    border-left:1px dotted #eee;
    margin-right:0;
}
ul#toplinks-for-user li a {
    color:#ccc; 
}

a.navOn {
    -moz-border-radius: 3px;
    -webkit-border-radius: 3px;
    padding:2px;
}

div#pagenav {
    font-size: .9em;
    clear: both;
    padding: 4px 15px;
    background: #e0e0e0;
    border-top: 1px solid #bbb;
    border-bottom: 1px solid #bbb;
}

div#pagenav ul li {
    padding: 2px 14px 2px 0;
    display: inline;
}

div#pagenav ul li a {
    text-decoration: none;
}

div#pagenav ul li a:hover {
    text-decoration: underline;
}

div#pageoptions {
    padding: 5px 14px;
    background-color: #ffe5cc;
}

div#pageoptions ul li {
    display: inline;
    padding: 5px 14px 5px 0;
    color: black;
    font-size: 1.1em;
}

div#pageoptions a {
    color: green;
}

div#pageoptions a.active {
    color: #ceebb6;
    background: green;
} #categories {
    background: #ffffff;
} #allPages {
    float: left;
    width: 40%;
    margin-right: 20px;
} #wantedPages {
    float: left;
    width: 55%;
} .wantedBy {
    color: #999;
} #tags {
    background: #e0e0e0;
    font-size: 1.1em;
    color: #676767;
    padding: 5px 13px;
    border-top: 1px solid #bbb;
    border-bottom: 1px solid #bbb;
} .showdiv {
    background: white;
    font-size: 11px;
    padding-left: 13px;
    padding-bottom: 5px;
    padding-top: 5px;
    font-weight: bold;
} 

#content {
    overflow: hidden;
    background: white;
    padding: 2em 13px;
    overflow: hidden;
} 

#content strong, b {
    font-weight: bold;
} #content em, i {
    font-style: italic;
} #content ul, #content ol {
    padding-left: 1.5em;
}

/* For IE6, which doesn't support child selectors */ #content ul li {
    list-style: disc;
}

/* this means that the last li in `ol li ul li` will end up with list-style: decimal instead of 'square' */ #content ol li {
    list-style: decimal;
}

/* The two direct child selectors below take care of that nesting problem, but only for Standards-Compliant Browsers */ #content ul > li {
    list-style: disc;
} #content ol > li {
    list-style: decimal;
} #content li {
    margin-left: 1.5em;
    margin-bottom: 5px;
}

/* tables */ #content table {
    border-collapse: collapse;
    border: 2px solid #bbb;
    background: #fff;
    margin-bottom: .6em;
} #content table td, #content table th {
    border: 1px dotted #aaa;
    padding: .5em;
} #content table thead th, #content table tfoot th {
    font-weight: bold;
    background: #f6f6f6;
} #content table tbody th, #content table tbody td {
    vertical-align: top;
    text-align: left;
} #content table tfoot td {
    border: 2px solid #000;
} #content table tr.odd td, #content table tr.odd th {
    background: #efefef;
} 

#bottomnav {
    background: #666;
    padding: 6px 8px;
    color: #000000;
    margin: 0;
    font-weight: bold;
    height: 2.8em;
    font-size: 0.9em;
    border-top: 2px solid #888;
} 

#bottomnav a {
    color:#afd1c6;
}

#bottomnav input {
    font-size: 1em;
}

li#site_settings {
    clear: left;
} .formdiv {
    text-align: left;
    background: #ffffff;
    padding-left: 13px;
    padding-bottom: 5px;
}

p {
    line-height: 1.3em;
    margin: 1.2em 0;
} .hide {
    display: none;
} .clickable {
    cursor: pointer;
} .center, #content .center, #content table .center {
    text-align: center;
} 

#powered {
    border-top: 2px solid #444;
    width: 100%;
    margin: 0;
    padding: 0.5em 0;
    text-align: center;
    background:#222;
} 

#content .syntax_help table {
    width: 100%;
} #content .syntax_help table th, #content .syntax_help table td {
    padding: 0.2em;
} .toc {
    background-color: #fffff3;
    border: 1px solid #dddddd ;
} .toc a {
    text-decoration: none;
}

