body {
  font-family: verdana, sans-serif;
  font-size: .8em;  
  background: rgb(234,234,234) url("../../images/layout/body_bg.png") center repeat-y;
  margin: 0;
  color: rgb(30,30,30);
}

a {
  /*color: rgb(0,51,204);*/
  color: #b00;
  text-decoration: none;
}

a:link {
  color: #b00;
}

a:visited {
  color: #b00;
  /*color: rgb(68,68,68);*/
}

a:hover {
  color: rgb(0,0,0);
  text-decoration: underline;
}

img {
  border: none;
}

p {
  margin-top: 0;
}

h1 a, h2 a, h3 a,
h1 a:visited, h2 a:visited, h3 a:visited {
  color: rgb(153,153,153);
}

h1 a:hover,
h2 a:hover,
h3 a:hover {
  color: rgb(0,0,0);
}

h1 {
  margin-top: .4em;
  font-size: 1.5em;
}

h2 {
  font-size: 1.2em;
  font-weight: bold;
}

h3 {
  font-size: 1em;
  font-weight: bold;
}

h4 {
  font-size: 1.2em;
  font-weight: normal;
}

hr {
  height: 1px;
  border-top: solid rgb(220,220,220);
  border-bottom: solid rgb(255,255,255);
  border-width: 1px 0 0 0;
}

table {
  border-collapse: collapse;
}

th {
  text-align: left;
}

ul {
  padding: 0; 
  margin: 0;
}

.content ul li {
  list-style-type: none;
  padding-left: 1.5em;
  background: url("../../images/global/list_bullet.png") top left no-repeat;
  background-position: .55em .55em;
}


/* Generic content classes
--------------------------------------- */

.hidden {
  clear: both;
  visibility: hidden;
}

h2.icon_control {
  background: url("../../images/global/icon_control.png") top left no-repeat;
}

h2.icon_model {
  background: url("../../images/global/icon_model.png") top left no-repeat;
}

h2.icon_view {
  background: url("../../images/global/icon_view.png") top left no-repeat;
}

h2.icon_control,
h2.icon_model,
h2.icon_view {
  margin-top: 1.2em;
  margin-bottom: .2em;
  padding: 15px 0 8px 55px;
  background-position: 0 5px;
}
