@import "../department/home.html";

#wrapper {
  width: 860px;
  margin-left: auto;
  margin-right: auto;
}

/* header
--------------------------------------- */

#header {
  height: 123px;
  background: url("../../images/layout/header_bg_catalyst.png") no-repeat;
  border: solid 1px rgb(255,255,255);
  border-bottom: solid 1px rgb(238,238,238);
}

.control #header {
  background: url("../../images/layout/header_bg_control.png") no-repeat;
}

.documentation #header {
  background: url("../../images/layout/header_bg_documentation.png") no-repeat;
}

.model #header {
  background: url("../../images/layout/header_bg_model.png") no-repeat;
}

.view #header {
  background: url("../../images/layout/header_bg_view.png") no-repeat;
}

#header h1 {
  position: relative;
  top: 15px;
  left: 20px;
  margin: 0;
  width: 170px;
  height: 58px;
  background: url("../../images/layout/catalyst_170pix.png");
}

#header h1 a {
  display: block;
  width: 170px;
  height: 58px;
}

#header h1 img {
  visibility: hidden;
}

#header form {
  float: right;
  margin: -30px 20px 0 0;
}

#header .submit {
  visibility: hidden;
  position: absolute;
}

/* menu_global
--------------------------------------- */

#menu_global {
  position: relative;
  top: 38px;
  left: 10px;
  float: left;
  padding: 0;
  margin: 0;
}

#menu_global li,
#menu_global li a {
  display: block;
  list-style-type: none;
}

#menu_global li {
  float: left;
  height: 29px;
  margin-right: 2px;
}

#menu_global a {
  padding: 8px 10px 5px 10px;
  text-decoration: none;
  color: rgb(85,85,85);
}

#menu_global li a:hover {
  color: rgb(0,0,0); 
}

.home #menu_global #home,
.community #menu_global #community,
.documentation #menu_global #documentation,
.developer #menu_global #developer,
.download #menu_global #download,
.support #menu_global #support {
  background: rgb(255,255,255) url("../../images/layout/menu_global_bg_left.png") top left no-repeat;
}

.home #home div,
.community #community div,
.documentation #documentation div,
.developer #developer div,
.download #download div,
.support #support div {
  cursor: default;
  color: rgb(85,85,85);
  background: url("../../images/layout/menu_global_bg_right.png") top right no-repeat;
}

html>body #menu_global div { 
  /* hidden from IE */
  height: 29px;  
}

/* test */


.home #home span,
.community #community span,
.documentation #documentation span,
.developer #developer span,
.download #download span,
.support #support span {
  cursor: default;
  display: block;
  background: url("../../images/layout/menu_global_bg_right.png") top right no-repeat;
}

.home #home span a,
.community #community span a,
.documentation #documentation span a,
.developer #developer span a,
.download #download span a,
.support #support span  a {
  color: rgb(0,0,0);
}

html>body #menu_global span { 
  /* hidden from IE */
  height: 29px;  
}



/* crumbtrail
--------------------------------------- */

#crumbtrail {
  margin: 10px 20px 5px 0;
  font-size: x-small;
}

#crumbtrail,
#crumbtrail a {
  color: rgb(153,153,153);
}

#crumbtrail a:hover {
  color: rgb(0,0,0);
}

/* main
--------------------------------------- */

#main {
  /*border-top: solid 1px rgb(220,220,220);*/
  padding: 0 20px 0 20px;
}

/* content
--------------------------------------- */

.content {
  float: left;
  width: 540px;
}

.fullwidth {
  width: 800px; 
}

/* catch
--------------------------------------- */

.catch {
  height: 258px;
  width: 480px;
  margin-bottom: 10px;
  padding: 20px 320px 20px 20px;
  color: rgb(51,51,51);
  font-size: 1.3em;
}

.catch h1 {
  margin-top: .5em;
  font-family: verdana, sans-serif;
  color: rgb(30,30,30);
} 

/* footer
--------------------------------------- */

#footer {
  height: 30px;
  margin: 0 20px 10px 20px;
  padding: 10px 0 0 0;
  clear: both;
  border-top: solid 1px rgb(220,220,220);
  color: rgb(153,153,153);
  font-size: x-small;
}


/* generic layout classes
--------------------------------------- */

.box {
  width: 180px;
  margin-top: 0;
  background: url("../../images/global/box_bg_gray.png") top right no-repeat; 
}

.left {
  float: left;
}

.middle {
  float: left;
  margin-left: 20px;
}

.right {
  float: right;
}

.box dt {  
  padding: 13px 10px 15px 10px;
  font-size: 1.1em;
  font-weight: bold;
  background: url("../../images/global/box_bg_left.png") top left no-repeat;
}

.development dt {
  padding-left: 54px;
  background-image: url("../../images/global/box_icon_development.png");
}

.deployment dt {
  padding-left: 54px;
  background-image: url("../../images/global/box_icon_deployment.png");
}

.scalability dt {
  padding-left: 54px;
  background-image: url("../../images/global/box_icon_scalability.png");
}

.box dd {
  margin-left: 0;
  padding: 0 10px 5px 10px;
}

.pod a img {
  vertical-align: middle;
}
