/* @override
    http://mojomojo.org:3000/.static/mojomojo.css
    http://localhost:3000/.static/mojomojo.css
*/

@import "reset.css";
/* @group Headlines */

.logintext {
    font-weight: bold;
    font-size: 1.25em;
    margin: 1em 0 1.5em 13px;
}

h1 {
    font-size: 2.5em;
}
#content h1 {
    margin-top: 0;
    font-size: 2em;
}
h2 {
    font-size: 1.4em;
}
h3 {
    font-size: 1.2em;
}
h4 {
    font-size: 1.1em;
}
h5, h6 {
    font-size: 1em;
}

.toc {
    display: inline-block;
    background-color: #F6F6F6;
    border: 1px solid #AAAAAA;
    padding: 1em;
    margin: 1em;
}

.toc ul {
    margin: 0;
    margin-left: -1em;
    padding: 0;
}
.toc > ul {
    margin-left: -2em;
}

.toc li {
    margin: 0;
    padding: 0;
    font-size: 1em !important;  /* some rule makes the font smaller as the indentation increases */
}

/* @end */

#htmltagcloud span {
    margin: 0 0.4em;
}

/* @group Links */

a, a:visited {
    color: #2274A8;
    text-decoration: underline;
}

#content a:visited {
    color: #996699;  /* per accessibility guidelines. See http://universalusability.com/access_by_design/links/differentiate.html */
}

a:hover, #content a:hover {
    color: #004276;
}



/* @end */

/* @group Navigation */

form { margin: 0;}

.navigation {
    color: #999;
}

a.navOn {
    color: #3a3a3a;
    background: #fff;
    font-weight: bold;
    text-decoration: none;
}

ul#toplinks-for-user {
    float: right;
}

.inputBox {
    background-color: #eee;
    padding: 5px;
    margin-bottom: 20px;
}

blockquote {
    display: block;
    width: 80%;
    background-color: #EEE;
    border : 1px dotted black;
    padding-right: 2em;
    padding-left: 2em;
    font-style: italic;
    quotes: "\201C" "\201D";
}
blockquote:before, blockquote:after {
    display: block;
    color: #7aa5d5;
    font-size: 5em;
}
blockquote:before {
    content: open-quote;
    height: 0px;
    margin-left: -0.45em;
}
blockquote:after {
    content: close-quote;
    height: 0.6em;
    margin-left: 99.5%;
    margin-top: -0.5em;
}

pre {
    width: 90%;
    background-color: #eee;
    padding: 10px;
    margin-left: 3em;
    margin-right: 1em ;
    overflow: auto
}

ol.setup {
    font-size: 1.5em;
    font-family: georgia, verdana, sans-serif;
    padding-left: 25px;
}

ol.setup li {
    margin-bottom: 20px
}

.byline {
    color: #999;
    padding-left: 4px;
    text-align: right;
    margin-bottom: 0px;
}

p#message {
    background: #eee;
    padding: 0.7em;
    border: 1px solid silver;
    margin: 1em 0;
}

#previewHeading {
    clear: left;
}

div#content_preview {
    clear: left;
    /* max-width: 920px; */
    border: 1px solid #ddd;
    padding: 5px;
    overflow: auto;
    margin: 0 0 1em 0;
    float: left;
    width: 49%;
}
div#edit_form {
    float: left;
    width: 49%;
}

textarea#body {
    /* max-width: 930px; */
    width: 100%;
}

textarea#comment {
    height: 150px;
    width: 400px;
}

#hidden_info, #edithelp {
    display: none;
    position: absolute;
    z-index: 1;
    width: 800px;
    overflow: auto;
}

div.tab-nav,
.syntax_help {
    background: #ddd;
    border: 1px solid #aaa;
    padding: 1em;
}

div.tab-nav {
    background: #ccc;
    border-bottom: 0;
    margin: 0;
    padding: 0 1em;
    height: 1.7em;
}

div.tab-nav a {
    outline: none;
    text-decoration: none;
    float: left;
    list-style: none;
    margin: 0 0.3em;
    padding: 0.2em 0.6em;
}

div.tab-nav a.active {
    background: #ddd;
    color: #222;
}

div.tab-nav a.close-button {
    position: absolute;
    right: 8px;
}

.syntax_help table {
    margin-bottom: 0;
    border-spacing: 0;
}


.syntax_help table+h3 {
    margin-top: 11px;
}

.syntax_help table th {
    font-size: 120%;
    font-weight: bold;
}

.syntax_help  table td {
    padding: 3px;
    vertical-align: top;
    border-top: 1px dotted #ccc;
}

.syntax_help table td.arrow {
    padding-right: 5px;
    padding-left: 10px;
    color: #999;
}

.syntax_help table td.label {
    font-weight: bold;
    white-space: nowrap;
    padding-right: 15px;
    color: #000;
}

.syntax_help h3 {
    font-weight: bold;
    font-weight: normal;
    margin: 0 0 5px 0;
    padding: 5px 0 0 0;
}

.syntax_help h2 {
    margin-top: 0;
}

.syntax_help pre {
    line-height: 90%;
    font-size: 120%;
    margin-left: 0;
    padding: 5px;
    width: 80%;
}

.rightHandSide {
    float: right;
    width: 187px;
    margin-left: 10px;
    padding-left: 20px;
    border-left: 1px dotted #ccc;
}

.rightHandSide p {
}

.newsList {
    margin-top: 20px;
}

.newsList p {
    margin-bottom: 30px;
}

.add a {
    color: green;
    text-decoration: none;
    vertical-align: middle;
}

.add a:hover {
    color: #003800;
}
a.tag {
    text-decoration: none;
}
a.tag:hover {
    text-decoration: underline;
}
.remove a {
    color: red;
    font-size: 1.1em;
    margin: 0px 1px;
    text-decoration: none;
}

.remove a:hover {
    color: #650000;
}

.tagedit {
    float: right;
}

#taginput {
    width: 120px;
}

.newWikiWord {
    color: #e33;
    background: lightyellow;
}

.newWikiWord a {
   /*  vertical-align: super;; */
    background: #f6d8b4;
    border: 1px solid #dddddd;
    font-weight: bold;
    text-decoration: none;
    color: #e33;
    padding: 0 2px;
    margin: 0 0 0 .1em;
}
#content .newWikiWord a:hover {
    background: #e33;
    color: #fff;
}

.protoPage a:link {
    color: #ab0b0b;
}

.protoPage a:visited {
    color: #666;
}

.protoPage a:hover {
    color: #fff;
    background-color: #ab0b0b;
}

/* @end */

/* @group Highlighted keywords in search */

.quoted {
    font-weight: bold;
}

span#search_hit_score {
    font-style: italic;
    font-size: smaller;
}

/* @end */

/* @group Pagination Links */

#content div.pager {
    border: 1px solid #eee;
    padding: .25em;
    text-align: center;
    font-size: 1.25em;
}

#content div.pager ul {
    list-style: none;
    margin: 0;
    padding: 0;
    font-weight: bold;
    list-style-position: inside;
}

#content div.pager .prev { margin: 0; padding: 0; float: left; }
#content div.pager .next { margin: 0; padding: 0; float: right; }

#content div.pager ul li {
    display: inline;
    margin: 0;
    padding: 0;
}

div.pager ul li a {
    text-decoration: none;
    padding: 2px .25em;
}
#content div.pager ul li a:hover {
    color: #fff;
    background: #2274a8;
}

#revisions {
    width: 100%;
}

dl#pageInfoMeta {
    padding: 0 0 0 1em;
    font-family: Arial,Helvetica,sans-serif;
}
dl#pageInfoMeta dt {
    line-height: 16px;
    width: 150px;
    font-weight: bold;
    float: left;
    clear: left;
    margin: 0;
    padding: 0;
}
dl#pageInfoMeta dd {
    line-height: 16px;
    width: 200px;
}

dl#authors {
    padding: 0;
    margin: 1.5em 0;
}
dl#authors dd,
dl#authors dt {
    line-height: 16px;
}

dl#authors dt {
    width: 200px;
}

/* @end */

/* @group Amazon Plugin */

div.amazon {
    width: 80%;
    padding-left: 2em;
    margin: 1em 4em 1em 4em;
    border: 1px solid #ccc;
    background: #eee;
}

div.amazon img {
    margin: 1.2em;
}

/* @end */
/* @group Lists */

table.recent {
    width: 100%;
}
table.recent .page          { width: 50%; }
table.recent .editedBy      { width: 20%; }
table.recent .timeEdited    { width: 20%; }
table.recent .actions       { width: 10%; }

/* @end */

/* @group Diff */

.diff {
    margin-right: 2em;
    overflow: visible;
    margin-top: 1em;
}

.diffdel, del {
/*    background: #bbb; 
    color: white; */
    border: dotted thin #fbbc08;
    color: #555;
    text-decoration: line-through;
    padding-left: 1px;
    padding-right: 1px;
}

.diffins, ins {
    border: dotted thin #fbbc08;
    background: yellow;
    color: black;
    text-decoration: none;
    padding-left: 1px;
    padding-right: 1px;
}

/* @end */

.form dl {
   margin: 13px;
   float: left;
}

.form dt {
    float: left;
    width: 10em;
    line-height: 24px;
}

.form dd {
    margin: 0 0 5px 90px;
    line-height: 24px;
    margin-left: 80px;
}

span.statusmsg {
    color: #335577;
    margin-left: 1em;
    font-size: 1.5em;
}

.login form input {
    width: 120px;
}

.thumb {
    border: none;
}

.addbutton {
    margin-right: 0.5em;
    background: white;
}

.thumb {
    background: #ddd;
    padding: 8px;
    margin: 8px;
}

.imageinfo {
    float: right;
    padding: 0px;
    margin: 0px;
    width: 12em;
    min-height: 250px;
}

.imageinfo dl {
    margin: 0;
}

.imageinfo h3 {
    margin-top: 0;
}

/* Transparent forms */

input.dual,textarea.dual {
    width: 500px;
    border: 1px solid #fff;
    margin: 0.1em;
    background: transparent;
    vertical-align: top;
}

input.dual:focus, textarea.dual:focus {
    border: 1px solid #000;
}

.dualbutton {
    display: none;
    margin: 0.1em;
}

big {
    font-size: 1.6em;
}

.framedpopup {
    background: #eee;
    padding: 3px;
    border: 1px solid silver;
    width: 440px;
}

dl {
    margin: 13px;
    font-family: sans-serif;
    clear: both;
}

/* Unused? */
dt {
    float: left;
    width: 80px;
    clear: left;
}

/* Unused? */
dd {
    margin: 0 0 5px 90px;
    margin-left: 80px;
}

div#comments div.comment dd {
    margin-left: 120px;
}

/* Common Forms Rules */

.standard_form .fields label {
    display: block;
    clear: both;
}
.standard_form .fields label span {
    text-align: right;
    margin-right: .5em;
}
.standard_form .fields label span,
.standard_form .fields label input {
    display: block;
    float: left;
    margin-bottom: 1ex;
}

/* Role Form */
.role_form .fields label span {
    width: 6em;
}

.role_form #member_input {
    width: 15em;
}

/* @group sidebar nav menu */

/* code shamelessly stolen from Eric Meyer's http://meyerweb.com/eric/css/edge/menus/demo.html */
/** with some slight modifications of colors, widths, etc. by Dave Naughton **/

#sidebar ul {
    padding: 0;
    margin: 0;
    border-bottom: 1px solid silver;
    font: 1em sans-serif;
}
#sidebar ul li {
    list-style-type: none;
    border: 1px solid silver;
    border-width: 1px 1px 0 3px;
    position: relative;
    margin: 0;
    padding: 0;
}
#sidebar ul ul {
    display: none;
}
#sidebar ul li:hover > ul {
    display: block;
    position: absolute;
    top: -1px;
    left: 100%;
}
#sidebar li a {
    display: block;
    padding: 5px 7px;
    text-decoration: none;
    background: #FFF;
}
#sidebar li.menuParent, li.menuParentMissingChildren > a {
    font-weight: bold;
}

#sidebar ul ul {width: 12em;}

#sidebar {
    position: absolute;
    right: 0;
    width: 10em;
    margin: 0;
    padding: 0;
    font-family: Arial, sans-serif;
}
#sidebar > ul {
    width: 11em;
    margin-left: -1px;
    font-size: 85%;
}
#sidebar ul {
    border: 1px solid silver;
    border-width: 0 0 0 1px;
}
#sidebar ul li {
    border-width: 1px 0;
    border-color: white;
    padding: 0 0 0 5px;
    line-height: 1.25em;
}
#sidebar ul ul {
    border-width: 0 1px 1px 1px;
    border-color: gray silver gray gray;
}
#sidebar ul ul li {
    border-color: #FEFEFC;
}
#sidebar li > a {
    background-color: transparent;
    padding: 3px;
}
#sidebar li:hover {
    background-color: #ddf;
}
#sidebar li.menuParent:hover {
    margin-left: -12.2em;
    border: 1px solid gray;
    background: #D7D6D4;
}
#sidebar li.menuParent:hover > a {
    background: inherit;
}
#sidebar li.menuParent:hover > ul {
    top: 1.75em;
    left: -1px;
    background: #FEFEFC;
}

ul#menuHome {
    border-bottom: 1px solid silver;
    font-weight: bold;
}

/* end of shamelessly stolen code */
/* @end */

.spinner {
    float: right;
    height: 1em;
}

#editstatus {
    color: red;
}

/* color values for errors copied from those for form elements */
img[defang_src] {
    border: 1px solid #ff0000;
    background-color: #ffdddd;
    color: #ff0000;
}

img[defang_src]:after {
    content: "External image forbidden due to its URL. Please contact your wiki administrator.";
}

div.photo img{
    margin: 1em;
    padding: 5px;
    border: 1px solid;
    border-color: #ccc #666 #666 #ccc
}

.end {
    clear: both;
}


.inline p,.inline dt,.inline dd{
    font-size: 0.8em;
    width: auto;
}

acronym {
    border-bottom: 1px dotted !important;
}

a.footnote {
    vertical-align: .6em;
    font-size: 75%;
}
a.footnote:before {
    content: "["
}
a.footnote:after {
    content: "]"
}

.inplace_field, .inplace_save, .inplace_cancel {
    font-size: 1em;
}

#title {
    margin-left: 1em;
}

#description {
    font-style: italic;
    margin-left: 2em;
}

#content img.right {
    margin-left: 1em;
    float: right;
}

#content img.left {
    margin-left: 1em;
    float: left;
}

#searchField {
    padding: 3px;
    font-size: 1em;
    border: 1px solid #bbb;
    margin-left: 10px;
}

#attachments table {
    width: 100%;
}

#attachments table #filename-th {
    width: 25%;
}

#attachments table #type-th {
    width: 25%;
}

#attachments table #size-th {
    width: 8%;
}

#attachments table #date-th {
    width: 12%;
}

#attachments table #actions-th {
    width: 30%;
}

#attachments table #actions {
    margin: 0;
}

#attachments table #actions li {
    margin-left: 0;
    padding-left: 0;
}

/* Copied from HTML-FormFu examples/vertically-aligned/vertically-aligned.css */

fieldset {
    padding: 1em;
}

fieldset .button,
fieldset .checkbox,
fieldset .contentbutton,
fieldset .date,
fieldset .file,
fieldset .image,
fieldset .multi,
fieldset .password,
fieldset .radio,
fieldset .recaptcha,
fieldset .reset,
fieldset .select,
fieldset .submit,
fieldset .text,
fieldset .textarea
{
    display: block;
    clear: left;
    border: 0;
    margin: 1px;
    /* when no label */
    margin-left: 12em;
}

/* make the noscript textarea sit below the iframe */
fieldset .recaptcha noscript textarea {
    display: block;
}

/* override the iframe's height="" - it seems a bit big */
fieldset .recaptcha iframe {
    height: 230px ! important;
}

fieldset .button.label,
fieldset .checkbox.label,
fieldset .contentbutton.label,
fieldset .date.label,
fieldset .file.label,
fieldset .image.label,
fieldset .multi.label,
fieldset .password.label,
fieldset .radio.label,
fieldset .recaptcha.label,
fieldset .reset.label,
fieldset .select.label,
fieldset .submit.label,
fieldset .text.label,
fieldset .textarea.label
{
    margin-left: 1px;
}

fieldset .error.label {
    /* border + margin swap values with above, to ensure rows align */
    border: 1px #fff;
    margin: 0;
}

form .form_error_message {
    padding: 0.5em;
    border: 1px solid #ff0000;
    background-color: #ffdddd;
}

fieldset .error_message {
    display: block;
    color: #ff0000;
}

fieldset .label .error_message {
    /* padding-left eq label width + padding-right */
    padding-left: 12em;
}

fieldset .error input,
fieldset .error textarea,
fieldset .error select {
    background-color: #ffdddd;
}

fieldset .button label,
fieldset .contentbutton label,
fieldset .checkbox label,
fieldset .date label,
fieldset .file label,
fieldset .image label,
fieldset .multi label,
fieldset .password label,
fieldset .radio label,
fieldset .radiogroup label,
fieldset .recaptcha label,
fieldset .select label,
fieldset .text label,
fieldset .textarea label
{
    display: inline;
    float: left;
    width: 11em;
    text-align: right;
    padding-right: 1em;
}

fieldset .radiogroup span label {
    /* undo the above style */
    float: none;
    width: auto;
    text-align: left;
    padding-right: 0;
}

fieldset .multi .elements {
    display: block;
    float: left;
}

fieldset .multi .elements label {
    display: block;
    width: auto;
    padding-right: 0.25em;
}

fieldset .multi input,
fieldset .multi select {
    display: block;
    float: left;
    margin-right: 0.5em;
}

fieldset.checkboxgroup,
fieldset.radiogroup
{
    margin: 0;
    margin-left: 12em;
    padding: 0;
    width: auto;
}

fieldset.radiogroup.label {
    border: 0;
    margin-left: 0em;
}

fieldset .comment .comment {
    /* when no label */
    display: block;
    margin-left: 0em;
}

fieldset .label .comment {
    display: block;
    margin-left: 12em;
}

/*** Alternative Layouts ***/

fieldset .notes {
    float: right;
    width: 30%;
    border: 1px dotted;
}

fieldset .multi.vertical input,
fieldset.checkboxgroup .vertical input,
fieldset.radiogroup .vertical input
{
    display: block;
    float: left;
    clear: left;
    margin-right: 0.5em;
}

fieldset .multi.vertical select {
    display: block;
    float: left;
    clear: right;
}

fieldset.checkboxgroup .vertical label,
fieldset.radiogroup .vertical label
{
    display: block;
    clear: right;
}

fieldset.radiogroup .vertical .subgroup {
    float: left;
}

fieldset .fullwidth label
{
    display: block;
    float: left;
    width: auto;
    text-align: left;
}

fieldset .fullwidth .error_message {
    padding-left: 0em;
}

fieldset .fullwidth textarea {
    display: block;
    clear: left;
    width: 30em;
}

fieldset .dojoeditor2 .RichTextEditable { /* Dojo::Editor2 */
    display: inline;
    float: left;
    background-color: #ffc;
    padding-bottom: 0.1em;
}

/* end of examples/vertically-aligned/vertically-aligned.css */

fieldset.secondCol {
    float: left;
    width: 45%;
}

#powered {
    float: right;
    clear: both;
    margin: 1em 0 0 0;
}

/* Used in the .profile page */
dl.list {
    font-family: Arial,Helvetica,sans-serif;
}
dl.list dt {
    font-weight: bold;
    width: 15em;
}
dl.list dd {
    margin-left: 16em;
}
#editlink {
    margin-left: 1em;
    font-weight: bold;
}




div#tags dl {
    margin: 0;
    padding: 0;
}

div#tags dt,
div#tags dd {
    margin: 0;
    padding: 0;
    height: 1.6em;
}

div#tags dt {
    font-size: .8em;
    font-weight: bold;
    width: 120px;
    clear: left;
}

div#bottomnav { clear: both; }

#revision_info {
    background: #ffa;
    padding: .25em .5em;
    color: #f60;
    font-size: 1.1em;
    float: right;
}

.attachment {
    background: #eee;
    padding: 3px;
    border: 1px solid silver;
    width: 640px
}

#progressbar {
    display: none;
    height: 18px;
    background-color: gray;
    margin: 10px 0;
}
#progressbar #progress {
    width: 0%;
    height: 18px;
    background-color: white;
    color: black;
}

/* Syntax::Highlight::Engine::Kate */
.kateAlert { color: #0000ff; }
.kateBaseN { color: #007f00; }
.kateBString { color: #c9a7ff; }
.kateChar { color: #ff00ff; }
.kateComment { color: #7f7f7f; }
.kateDataType { color: #0000ff; }
.kateDecVal { color: #00007f; }
.kateError { color: #ff0000; }
.kateFloat { color: #00007f; }
.kateFunction { color: #007f00; }
.kateIString { color: #ff0000; }
.kateOperator { color: #ffa500; }
.kateOthers { color: #b03060; }
.kateRegionMarker { color: #96b9ff; }
.kateReserved { color: #9b30ff; }
.kateString { color: #ff0000; }
.kateVariable { color: #0000ff; }
.kateWarning { color: #0000ff; }

#editspinner { display: none; }
#search_results {
    text-indent: 1em;
}
#search_results .search_result {
    list-style-type: none;
}

.display-none { display: none; }
.float-left { float: left; }
.float-right { float: right; }

table.roles {
    min-width: 50%;
    margin-top: 2em;
}

form.site-configuration .submit {
    position: relative;
    left: -45%;
    padding-top: 2em;
}

li.tabnav {
}

.error_detail {
    display: block;
    margin-left: 1em;
    font-style: italic;
}

.message_body {
    font-size: 2em;
    line-height: 1.666em;
}

#nonexistent_pages {
    font-size: 1.666em; 
    background-color: lightyellow; 
    border: 1px solid black; 
    padding: 5px; 
    max-width: 25em;
}
